package net.lensfrex.dscape.auth;

import cn.dev33.satoken.stp.StpInterface;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.dscape.dao.entity.RolePermission;
import net.lensfrex.dscape.dao.entity.UserRole;
import net.lensfrex.dscape.dao.service.RolePermissionService;
import net.lensfrex.dscape.dao.service.UserRoleService;
import net.lensfrex.dscape.dao.service.impl.UserRoleServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Component
public class StpInterfaceImpl implements StpInterface {

    private final UserRoleService userRoleService;
    private final RolePermissionService rolePermissionService;

    private final RedisTemplate<String, String> redis;

    @Autowired
    public StpInterfaceImpl(RedisTemplate<String, String> redis,
                            UserRoleService userRoleService,
                            RolePermissionService rolePermissionService) {

        this.userRoleService = userRoleService;
        this.rolePermissionService = rolePermissionService;
        this.redis = redis;
    }

    @Override
    public List<String> getPermissionList(Object loginId, String loginType) {
        String uid = (String) loginId;

        List<String> userRoles = this.getRoleList(loginId, loginType);
        List<String> rolePermissions = new ArrayList<>();

        for (String role : userRoles) {
            String redisRolePermissionKey = "dscape:auth:role:permission:" + role;

            if (Boolean.FALSE.equals(redis.hasKey(redisRolePermissionKey))) {
                log.debug("角色 " + uid + " 的权限信息缓存不存在，从数据库中查找并放入缓存");

                rolePermissions.addAll(rolePermissionService.getPermissions(role));
                if (rolePermissions.isEmpty()) {
                    return rolePermissions;
                }

                redis.opsForList().leftPushAll(redisRolePermissionKey, rolePermissions);
            } else {
                List<String> cachedUserRole = redis.opsForList().range(redisRolePermissionKey, 0, -1);
                if (cachedUserRole == null) {
                    return rolePermissions;
                }

                rolePermissions.addAll(cachedUserRole);
            }
        }

        return rolePermissions;
    }

    @Override
    public List<String> getRoleList(Object loginId, String loginType) {
        String uid = (String) loginId;
        String redisUserRoleKey = "dscape:auth:user:role:" + uid;

        List<String> userRoleList = new ArrayList<>(2);
        if (Boolean.FALSE.equals(redis.hasKey(redisUserRoleKey))) {
            log.debug("用户 " + uid + " 的角色信息缓存不存在，从数据库中查找并放入缓存");

            userRoleList.addAll(userRoleService.getRoles(uid));

            if (userRoleList.isEmpty()) {
                return userRoleList;
            }

            redis.opsForList().leftPushAll(redisUserRoleKey, userRoleList);
        } else {
            List<String> cachedUserRole = redis.opsForList().range(redisUserRoleKey, 0, -1);
            if (cachedUserRole == null) {
                return userRoleList;
            }

            userRoleList.addAll(cachedUserRole);
        }

        return userRoleList;
    }
}