/*
 * Class created by lensfrex.
 */

package net.lensfrex.dscape.web.service.user;

import cn.dev33.satoken.stp.SaTokenInfo;
import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.dscape.dao.entity.UserBasic;
import net.lensfrex.dscape.dao.service.UserBasicService;
import net.lensfrex.dscape.dto.request.UserLoginRequestBody;
import net.lensfrex.dscape.dto.request.user.RegisterRequestBody;
import net.lensfrex.dscape.dto.response.general.ResponseCode;
import net.lensfrex.dscape.dto.response.user.RegisterResponseBody;
import net.lensfrex.dscape.enums.user.UserStatusEnum;
import net.lensfrex.dscape.exception.GlobalException;
import net.lensfrex.dscape.utils.ObjectJsonSerializer;
import org.mindrot.jbcrypt.BCrypt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class UserService {

    private static final String USER_INFO_CACHE_KEY = "dscape:user:basic";

    private final ObjectJsonSerializer objectJsonSerializer;

    private final RedisTemplate<String, String> redis;

    private final UserBasicService userBasicService;

    @Autowired
    public UserService(ObjectJsonSerializer objectJsonSerializer,
                       RedisTemplate<String, String> redis,
                       UserBasicService userBasicService) {

        this.objectJsonSerializer = objectJsonSerializer;
        this.redis = redis;
        this.userBasicService = userBasicService;
    }

    public SaTokenInfo login(UserLoginRequestBody requestBody) {
        String username = requestBody.getUserName();
        String password = requestBody.getPassword();
        UserBasic userBasic;

        if (Boolean.TRUE.equals(redis.hasKey(USER_INFO_CACHE_KEY))) {
            String data = (String) redis.opsForHash().get(USER_INFO_CACHE_KEY, username);
            userBasic = objectJsonSerializer.deserialize(data, UserBasic.class);
        } else {
            QueryWrapper<UserBasic> wrapper = new QueryWrapper<>();
            wrapper.lambda()
//                    .select(UserBasic::getUid, UserBasic::getPassword, UserBasic::getUserName, UserBasic::getStatus)
                    .eq(UserBasic::getUserName, username)
                    .eq(UserBasic::getIsDeleted, 0);

            userBasic = userBasicService.getOne(wrapper);
            try {
                redis.opsForHash().put(USER_INFO_CACHE_KEY, username, objectJsonSerializer.serialize(userBasic));
            } catch (Exception e) {
                log.info("将用户数据存入redis缓存时发生错误", e);
            }
        }

        if (userBasic != null && BCrypt.checkpw(password, userBasic.getPassword())) {
            if (userBasic.getStatus() == UserStatusEnum.BANNED.getCode()) {
                throw new GlobalException(ResponseCode.USER_WAS_BANNED);
            } else if (userBasic.getStatus() == UserStatusEnum.NOT_IDENTIFIED.getCode()) {
                throw new GlobalException(ResponseCode.USER_WAS_NOT_IDENTIFIED);
            }

            StpUtil.login(requestBody.getUserName());
        } else {
            throw new GlobalException(ResponseCode.USERNAME_OR_PASSWORD_WRONG);
        }

        return StpUtil.getTokenInfo();
    }

    public RegisterResponseBody register(RegisterRequestBody requestBody) {
        return new RegisterResponseBody();
    }
}